@@ -13,8 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Union
+from typing import Type
+from typing import Optional
+
+import json
 import os
-import sys
 import ssl
 import socket
 import copy
@@ -23,16 +27,8 @@ import time
 
 from libcloud.utils.py3 import ET
 
-try:
-    import simplejson as json
-except:
-    import json
-
-import requests
-
 import libcloud
 
-from libcloud.utils.py3 import PY25
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlparse
 from libcloud.utils.py3 import urlencode
@@ -121,10 +117,16 @@ class Response(object):
     A base Response class to derive from.
     """
 
-    status = httplib.OK  # Response status code
-    headers = {}  # Response headers
-    body = None  # Raw response body
-    object = None  # Parsed response body
+    # Response status code
+    status = httplib.OK  # type: int
+    # Response headers
+    headers = {}  # type: dict
+
+    # Raw response body
+    body = None
+
+    # Parsed response body
+    object = None
 
     error = None  # Reason returned by the server.
     connection = None  # Parent connection class
@@ -154,7 +156,8 @@ class Response(object):
 
         if not self.success():
             raise exception_from_message(code=self.status,
-                                         message=self.parse_error())
+                                         message=self.parse_error(),
+                                         headers=self.headers)
 
         self.object = self.parse_body()
 
@@ -191,6 +194,7 @@ class Response(object):
         :return: ``True`` or ``False``
         """
         # pylint: disable=E1101
+        import requests
         return self.status in [requests.codes.ok, requests.codes.created,
                                httplib.OK, httplib.CREATED, httplib.ACCEPTED]
 
@@ -206,7 +210,7 @@ class JsonResponse(Response):
 
         try:
             body = json.loads(self.body)
-        except:
+        except Exception:
             raise MalformedResponseError(
                 'Failed to parse JSON',
                 body=self.body,
@@ -231,12 +235,11 @@ class XmlResponse(Response):
             except ValueError:
                 # lxml wants a bytes and tests are basically hard-coded to str
                 body = ET.XML(self.body.encode('utf-8'))
-        except:
+        except Exception:
             raise MalformedResponseError('Failed to parse XML',
                                          body=self.body,
                                          driver=self.connection.driver)
         return body
-
     parse_error = parse_body
 
 
@@ -270,6 +273,7 @@ class RawResponse(Response):
         :return: ``True`` or ``False``
         """
         # pylint: disable=E1101
+        import requests
         return self.status in [requests.codes.ok, requests.codes.created,
                                httplib.OK, httplib.CREATED, httplib.ACCEPTED]
 
@@ -278,11 +282,16 @@ class RawResponse(Response):
         if not self._response:
             response = self.connection.connection.getresponse()
             self._response = HttpLibResponseProxy(response)
-            self.body = response.content
             if not self.success():
                 self.parse_error()
         return self._response
 
+    @property
+    def body(self):
+        # Note: We use property to avoid saving whole response body into RAM
+        # See https://github.com/apache/libcloud/pull/1132 for details
+        return self.response.body
+
     @property
     def reason(self):
         if not self._reason:
@@ -299,11 +308,11 @@ class Connection(object):
     responseCls = Response
     rawResponseCls = RawResponse
     connection = None
-    host = '127.0.0.1'
+    host = '127.0.0.1'  # type: str
     port = 443
-    timeout = None
+    timeout = None  # type: Optional[Union[int, float]]
     secure = 1
-    driver = None
+    driver = None  # type:  Type[BaseDriver]
     action = None
     cache_busting = False
     backoff = None
@@ -347,16 +356,21 @@ class Connection(object):
 
     def set_http_proxy(self, proxy_url):
         """
-        Set a HTTP proxy which will be used with this connection.
+        Set a HTTP / HTTPS proxy which will be used with this connection.
 
         :param proxy_url: Proxy URL (e.g. http://<hostname>:<port> without
                           authentication and
-                          http://<username>:<password>@<hostname>:<port> for
-                          basic auth authentication information.
+                          <scheme>://<username>:<password>@<hostname>:<port>
+                          for basic auth authentication information.
         :type proxy_url: ``str``
         """
         self.proxy_url = proxy_url
 
+        # NOTE: Because of the way connection instantion works, we need to call
+        # self.connection.set_http_proxy() here. Just setting "self.proxy_url"
+        # won't work.
+        self.connection.set_http_proxy(proxy_url=proxy_url)
+
     def set_context(self, context):
         if not isinstance(context, dict):
             raise TypeError('context needs to be a dictionary')
@@ -438,9 +452,7 @@ class Connection(object):
         if not hasattr(kwargs, 'cert_file') and hasattr(self, 'cert_file'):
             kwargs.update({'cert_file': getattr(self, 'cert_file')})
 
-        # Timeout is only supported in Python 2.6 and later
-        # http://docs.python.org/library/httplib.html#httplib.HTTPConnection
-        if self.timeout and not PY25:
+        if self.timeout:
             kwargs.update({'timeout': self.timeout})
 
         if self.proxy_url:
@@ -481,7 +493,7 @@ class Connection(object):
         self.ua.append(token)
 
     def request(self, action, params=None, data=None, headers=None,
-                method='GET', raw=False, stream=False):
+                method='GET', raw=False, stream=False, json=None):
         """
         Request a given `action`.
 
@@ -587,6 +599,7 @@ class Connection(object):
                     url=url,
                     body=data,
                     headers=headers,
+                    raw=raw,
                     stream=stream)
             else:
                 if retry_enabled:
@@ -601,8 +614,7 @@ class Connection(object):
                 else:
                     self.connection.request(method=method, url=url, body=data,
                                             headers=headers, stream=stream)
-        except socket.gaierror:
-            e = sys.exc_info()[1]
+        except socket.gaierror as e:
             message = str(e)
             errno = getattr(e, 'errno', None)
 
@@ -619,8 +631,7 @@ class Connection(object):
                 raise socket.gaierror(msg)
             self.reset_context()
             raise e
-        except ssl.SSLError:
-            e = sys.exc_info()[1]
+        except ssl.SSLError as e:
             self.reset_context()
             raise ssl.SSLError(str(e))
 
@@ -875,7 +886,7 @@ class KeyCertificateConnection(CertificateConnection):
 
     def __init__(self, key_file, cert_file, secure=True, host=None, port=None,
                  url=None, proxy_url=None, timeout=None, backoff=None,
-                 retry_delay=None):
+                 retry_delay=None, ca_cert=None):
         """
         Initialize `cert_file`; set `secure` to an ``int`` based on
         passed value.
@@ -897,7 +908,7 @@ class ConnectionUserAndKey(ConnectionKey):
     Base connection class which accepts a ``user_id`` and ``key`` argument.
     """
 
-    user_id = None
+    user_id = None  # type: int
 
     def __init__(self, user_id, key, secure=True, host=None, port=None,
                  url=None, timeout=None, proxy_url=None,
@@ -916,7 +927,7 @@ class BaseDriver(object):
     Base driver class from which other classes can inherit from.
     """
 
-    connectionCls = ConnectionKey
+    connectionCls = ConnectionKey  # type: Type[Connection]
 
     def __init__(self, key, secret=None, secure=True, host=None, port=None,
                  api_version=None, region=None, **kwargs):
