@@ -421,7 +421,7 @@ class GoogleInstalledAppAuthConnection(GoogleBaseAuthConnection):
         if PY3:
             code = input('Enter Code: ')
         else:
-            code = raw_input('Enter Code: ')
+            code = raw_input('Enter Code: ')  # NOQA pylint: disable=undefined-variable
         return code
 
     def get_new_token(self):
@@ -617,10 +617,7 @@ class GoogleAuthType(object):
         """
         Checks S3 key format: alphanumeric chars starting with GOOG.
         """
-        return (
-            len(user_id) >= 20 and len(user_id) < 30 and user_id
-            .startswith('GOOG')
-        )
+        return user_id.startswith('GOOG')
 
     @staticmethod
     def _is_sa(user_id):
