@@ -360,16 +360,25 @@ class AzureNodeDriver(NodeDriver):
                                  ex_offer, ex_sku, ex_version)
             return i[0] if i else None
 
-    def list_nodes(self, ex_resource_group=None, ex_fetch_nic=True):
+    def list_nodes(self, ex_resource_group=None,
+                   ex_fetch_nic=True,
+                   ex_fetch_power_state=True):
         """
         List all nodes.
 
-        :param ex_resource_group: List nodes in a specific resource group.
-        :type ex_urn: ``str``
+        :param ex_resource_group: The resource group to list all nodes from.
+        :type ex_resource_group: ``str``
 
         :param ex_fetch_nic: Fetch NIC resources in order to get
-        IP address information for nodes (requires extra API calls).
-        :type ex_urn: ``bool``
+        IP address information for nodes.  If True, requires an extra API
+        call for each NIC of each node.  If False, IP addresses will not
+        be returned.
+        :type ex_fetch_nic: ``bool``
+
+        :param ex_fetch_power_state: Fetch node power state.  If True, requires
+        an extra API call for each node.  If False, node state
+        will be returned based on provisioning state only.
+        :type ex_fetch_power_state: ``bool``
 
         :return:  list of node objects
         :rtype: ``list`` of :class:`.Node`
@@ -385,7 +394,9 @@ class AzureNodeDriver(NodeDriver):
                      % (self.subscription_id)
         r = self.connection.request(action,
                                     params={"api-version": "2015-06-15"})
-        return [self._to_node(n, fetch_nic=ex_fetch_nic)
+        return [self._to_node(n,
+                              fetch_nic=ex_fetch_nic,
+                              fetch_power_state=ex_fetch_power_state)
                 for n in r.object["value"]]
 
     def create_node(self,
@@ -394,7 +405,7 @@ class AzureNodeDriver(NodeDriver):
                     image,
                     auth,
                     ex_resource_group,
-                    ex_storage_account,
+                    ex_storage_account=None,
                     ex_blob_container="vhds",
                     location=None,
                     ex_user_name="azureuser",
@@ -404,6 +415,7 @@ class AzureNodeDriver(NodeDriver):
                     ex_tags={},
                     ex_customdata="",
                     ex_use_managed_disks=False,
+                    ex_disk_size=None,
                     ex_storage_account_type="Standard_LRS"):
         """Create a new node instance. This instance will be started
         automatically.
@@ -502,6 +514,9 @@ class AzureNodeDriver(NodeDriver):
             in all regions (default False).
         :type ex_use_managed_disks: ``bool``
 
+        :param ex_disk_size: Custom OS disk size in GB
+        :type ex_disk_size: ``int``
+
         :param ex_storage_account_type: The Storage Account type,
             ``Standard_LRS``(HDD disks) or ``Premium_LRS``(SSD disks).
         :type ex_storage_account_type: str
@@ -509,6 +524,9 @@ class AzureNodeDriver(NodeDriver):
         :return: The newly created node.
         :rtype: :class:`.Node`
         """
+        if not ex_use_managed_disks and ex_storage_account is None:
+            raise ValueError("ex_use_managed_disks is False, "
+                             "must provide ex_storage_account")
 
         if location is None:
             location = self.default_location
@@ -615,6 +633,11 @@ class AzureNodeDriver(NodeDriver):
             }
         }
 
+        if ex_disk_size:
+            data['properties']['storageProfile']['osDisk'].update({
+                'diskSizeGB': ex_disk_size
+            })
+
         if ex_customdata:
             data["properties"]["osProfile"]["customData"] = \
                 base64.b64encode(ex_customdata)
@@ -623,7 +646,7 @@ class AzureNodeDriver(NodeDriver):
 
         if isinstance(auth, NodeAuthSSHKey):
             data["properties"]["osProfile"]["adminPassword"] = \
-                binascii.hexlify(os.urandom(20))
+                binascii.hexlify(os.urandom(20)).decode("utf-8")
             data["properties"]["osProfile"]["linuxConfiguration"] = {
                 "disablePasswordAuthentication": "true",
                 "ssh": {
@@ -631,7 +654,7 @@ class AzureNodeDriver(NodeDriver):
                         {
                             "path": '/home/%s/.ssh/authorized_keys' % (
                                 ex_user_name),
-                            "keyData": auth.pubkey
+                            "keyData": auth.pubkey  # pylint: disable=no-member
                         }
                     ]
                 }
@@ -680,7 +703,11 @@ class AzureNodeDriver(NodeDriver):
             else:
                 return False
 
-    def destroy_node(self, node, ex_destroy_nic=True, ex_destroy_vhd=True):
+    def destroy_node(self, node,
+                     ex_destroy_nic=True,
+                     ex_destroy_vhd=True,
+                     ex_poll_qty=10,
+                     ex_poll_wait=10):
         """
         Destroy a node.
 
@@ -695,12 +722,25 @@ class AzureNodeDriver(NodeDriver):
         this node (default True).
         :type node: ``bool``
 
-        :return: True if the destroy was successful, False otherwise.
+        :param ex_poll_qty: Number of retries checking if the node
+        is gone, destroying the NIC or destroying the VHD (default 10).
+        :type node: ``int``
+
+        :param ex_poll_wait: Delay in seconds between retries (default 10).
+        :type node: ``int``
+
+        :return: True if the destroy was successful, raises exception
+        otherwise.
         :rtype: ``bool``
         """
 
+        do_node_polling = (ex_destroy_nic or ex_destroy_vhd)
+
         # This returns a 202 (Accepted) which means that the delete happens
         # asynchronously.
+        # If returns 404, we may be retrying a previous destroy_node call that
+        # failed to clean up its related resources, so it isn't taken as a
+        # failure.
         try:
             self.connection.request(node.id,
                                     params={"api-version": "2015-06-15"},
@@ -708,21 +748,28 @@ class AzureNodeDriver(NodeDriver):
         except BaseHTTPError as h:
             if h.code == 202:
                 pass
+            elif h.code == 204:
+                # Returns 204 if node already deleted.
+                do_node_polling = False
             else:
-                return False
+                raise
 
-        # Need to poll until the node actually goes away.
-        while True:
+        # Poll until the node actually goes away (otherwise attempt to delete
+        # NIC and VHD will fail with "resource in use" errors).
+        retries = ex_poll_qty
+        while do_node_polling and retries > 0:
             try:
-                time.sleep(10)
+                time.sleep(ex_poll_wait)
                 self.connection.request(
                     node.id,
                     params={"api-version": RESOURCE_API_VERSION})
+                retries -= 1
             except BaseHTTPError as h:
-                if h.code == 404:
+                if h.code in (204, 404):
+                    # Node is gone
                     break
                 else:
-                    return False
+                    raise
 
         # Optionally clean up the network
         # interfaces that were attached to this node.
@@ -730,42 +777,44 @@ class AzureNodeDriver(NodeDriver):
             node.extra["properties"]["networkProfile"]["networkInterfaces"]
         if ex_destroy_nic:
             for nic in interfaces:
-                while True:
+                retries = ex_poll_qty
+                while retries > 0:
                     try:
-                        self.connection.request(
-                            nic["id"],
-                            params={"api-version": RESOURCE_API_VERSION},
-                            method='DELETE')
+                        self.ex_destroy_nic(self._to_nic(nic))
                         break
                     except BaseHTTPError as h:
-                        if h.code == 202:
-                            break
-                        inuse = h.message.startswith("[NicInUse]")
-                        if h.code == 400 and inuse:
-                            time.sleep(10)
+                        retries -= 1
+                        if (h.code == 400 and
+                                h.message.startswith("[NicInUse]") and
+                                retries > 0):
+                            time.sleep(ex_poll_wait)
                         else:
-                            return False
+                            raise
 
         # Optionally clean up OS disk VHD.
         vhd = node.extra["properties"]["storageProfile"]["osDisk"].get("vhd")
         if ex_destroy_vhd and vhd is not None:
-            while True:
+            retries = ex_poll_qty
+            resourceGroup = node.id.split("/")[4]
+            while retries > 0:
                 try:
-                    resourceGroup = node.id.split("/")[4]
-                    self._ex_delete_old_vhd(
-                        resourceGroup,
-                        vhd["uri"])
-                    break
+                    if self._ex_delete_old_vhd(resourceGroup, vhd["uri"]):
+                        break
+                    # Unfortunately lease errors usually result in it returning
+                    # "False" with no more information.  Need to wait and try
+                    # again.
                 except LibcloudError as e:
-                    if "LeaseIdMissing" in str(e):
+                    retries -= 1
+                    if "LeaseIdMissing" in str(e) and retries > 0:
                         # Unfortunately lease errors
                         # (which occur if the vhd blob
                         # hasn't yet been released by the VM being destroyed)
                         # get raised as plain
                         # LibcloudError.  Wait a bit and try again.
-                        time.sleep(10)
+                        time.sleep(ex_poll_wait)
                     else:
                         raise
+                time.sleep(10)
 
         return True
 
@@ -913,7 +962,7 @@ class AzureNodeDriver(NodeDriver):
         if ex_lun is None:
             # find the smallest unused logical unit number
             used_luns = [disk['lun'] for disk in disks]
-            free_luns = [lun for lun in range(0, 63) if lun not in used_luns]
+            free_luns = [lun for lun in range(0, 64) if lun not in used_luns]
             if len(free_luns) > 0:
                 ex_lun = free_luns[0]
             else:
@@ -951,6 +1000,53 @@ class AzureNodeDriver(NodeDriver):
             })
         return True
 
+    def ex_resize_volume(self, volume, new_size, resource_group):
+        """
+        Resize a volume.
+
+        :param volume: A volume to resize.
+        :type volume: :class:`StorageVolume`
+
+        :param new_size: The new size to resize the volume to in Gib.
+        :type new_size: ``int``
+
+        :param resource_group: The name of the resource group in which to
+            create the volume.
+        :type resource_group: ``str``
+
+        """
+        action = (
+            u'/subscriptions/{subscription_id}/resourceGroups/{resource_group}'
+            u'/providers/Microsoft.Compute/disks/{volume_name}'
+        ).format(
+            subscription_id=self.subscription_id,
+            resource_group=resource_group,
+            volume_name=volume.name,
+        )
+
+        data = {
+            'location': volume.extra['location'],
+            'properties': {
+                'diskSizeGB': new_size,
+                'creationData': volume.extra['properties']['creationData']
+            }
+        }
+
+        response = self.connection.request(
+            action,
+            method='PUT',
+            params={
+                'api-version': '2018-06-01',
+            },
+            data=data
+        )
+
+        return self._to_volume(
+            response.object,
+            name=volume.name,
+            ex_resource_group=resource_group
+        )
+
     def detach_volume(self, volume, ex_node=None):
         """
         Detach a managed volume from a node.
@@ -1230,13 +1326,14 @@ class AzureNodeDriver(NodeDriver):
         """
         if not isinstance(resource, basestring):
             resource = resource.id
-        self.connection.request(
+        r = self.connection.request(
             resource,
             method='DELETE',
             params={
                 'api-version': RESOURCE_API_VERSION
             },
         )
+        return r.status in [200, 202, 204]
 
     def ex_get_ratecard(self, offer_durable_id, currency='USD',
                         locale='en-US', region='US'):
@@ -1255,7 +1352,7 @@ class AzureNodeDriver(NodeDriver):
         :type locale: ``str``
 
         :param region: Region (two-letter code) (default: "US")
-        :type regions: ``str``
+        :type region: ``str``
 
         :return: A dictionary of rates whose ID's correspond to nothing at all
         :rtype: ``dict``
@@ -1322,14 +1419,14 @@ class AzureNodeDriver(NodeDriver):
 
         :param offer: The complete resource path to an offer (as returned by
         `ex_list_offers`)
-        :type publisher: ``str``
+        :type offer: ``str``
 
         :return: A list of tuples in the form
         ("sku id", "sku name")
         :rtype: ``list``
         """
 
-        action = "%s/skus" % (offer)
+        action = "%s/skus" % offer
         r = self.connection.request(action,
                                     params={"api-version": "2015-06-15"})
         return [(sku["id"], sku["name"]) for sku in r.object]
@@ -1407,10 +1504,11 @@ class AzureNodeDriver(NodeDriver):
         :type location: :class:`.NodeLocation`
         """
 
-        if location is None and self.default_location:
-            location = self.default_location
-        else:
-            raise ValueError("location is required.")
+        if location is None:
+            if self.default_location:
+                location = self.default_location
+            else:
+                raise ValueError("location is required.")
 
         target = "/subscriptions/%s/resourceGroups/%s/" \
                  "providers/Microsoft.Network/networkSecurityGroups/%s" \
@@ -1440,10 +1538,11 @@ class AzureNodeDriver(NodeDriver):
         :type location: :class:`.NodeLocation`
         """
 
-        if location is None and self.default_location:
-            location = self.default_location
-        else:
-            raise ValueError("location is required.")
+        if location is None:
+            if self.default_location:
+                location = self.default_location
+            else:
+                raise ValueError("location is required.")
 
         target = "/subscriptions/%s/resourceGroups/%s/" \
                  "providers/Microsoft.Network/networkSecurityGroups/%s" \
@@ -1510,7 +1609,7 @@ class AzureNodeDriver(NodeDriver):
                      (self.subscription_id, resource_group)
         r = self.connection.request(
             action,
-            params={"api-version": RESOURCE_API_VERSION})
+            params={"api-version": "2015-06-15"})
         return [self._to_nic(net) for net in r.object["value"]]
 
     def ex_get_nic(self, id):
@@ -1527,6 +1626,123 @@ class AzureNodeDriver(NodeDriver):
         r = self.connection.request(id, params={"api-version": "2015-06-15"})
         return self._to_nic(r.object)
 
+    def ex_update_nic_properties(self, network_interface,
+                                 resource_group, properties):
+        """
+        Update the properties of an already existing virtual network
+        interface (NIC).
+
+        :param network_interface: The NIC to update.
+        :type network_interface: :class:`.AzureNic`
+
+        :param resource_group: The resource group to check the ip address in.
+        :type resource_group: ``str``
+
+        :param properties: The dictionary of the NIC's properties
+        :type properties: ``dict``
+
+        :return: The NIC object
+        :rtype: :class:`.AzureNic`
+        """
+
+        target = "/subscriptions/%s/resourceGroups/%s/providers" \
+                 "/Microsoft.Network/networkInterfaces/%s" \
+                 % (self.subscription_id, resource_group,
+                    network_interface.name)
+
+        data = {
+            "properties": properties,
+            "location": network_interface.location,
+        }
+
+        r = self.connection.request(target,
+                                    params={"api-version": '2018-06-01'},
+                                    data=data,
+                                    method='PUT')
+        return AzureNic(r.object["id"], r.object["name"], r.object["location"],
+                        r.object["properties"])
+
+    def ex_update_network_profile_of_node(self, node, network_profile):
+        """
+        Update the network profile of a node. This method can be used to
+        attach or detach a NIC to a node.
+
+        :param node: A node to attach the network interface to.
+        :type node: :class:`Node`
+
+        :param network_profile: The new network profile to update.
+        :type network_profile: ``dict``
+        """
+        action = node.extra['id']
+        location = node.extra['location']
+        self.connection.request(
+            action,
+            method='PUT',
+            params={
+                'api-version': '2018-06-01'
+            },
+            data={
+                "id": node.id,
+                "name": node.name,
+                "type": "Microsoft.Compute/virtualMachines",
+                "location": location,
+                'properties': {
+                    'networkProfile': network_profile
+                }
+            })
+
+    def ex_destroy_nic(self, nic):
+        """
+        Destroy a NIC.
+
+        :param nic: The NIC to destroy.
+        :type nic: ``.AzureNic``
+
+        :return: True on success
+        :rtype: ``bool``
+        """
+
+        try:
+            self.connection.request(
+                nic.id,
+                params={"api-version": "2015-06-15"},
+                method='DELETE')
+            return True
+        except BaseHTTPError as h:
+            if h.code in (202, 204):
+                # Deletion is accepted (but deferred), or NIC is already
+                # deleted
+                return True
+            else:
+                raise
+
+    def ex_check_ip_address_availability(self, resource_group, network,
+                                         ip_address):
+        """
+        Checks whether a private IP address is available for use. Also
+        returns an object that contains the available IPs in the subnet.
+
+        :param resource_group: The resource group to check the ip address in.
+        :type resource_group: ``str``
+
+        :param network: The virtual network.
+        :type network: :class:`.AzureNetwork`
+
+        :param ip_address: The private IP address to be verified.
+        :type ip_address: ``str``
+        """
+        params = {"api-version": '2018-06-01'}
+        action = "/subscriptions/%s/resourceGroups/%s/providers" \
+                 "/Microsoft.Network/virtualNetworks/%s/" \
+                 "CheckIPAddressAvailability" % \
+                 (self.subscription_id, resource_group, network.name)
+        if ip_address is not None:
+            params["ipAddress"] = ip_address
+        r = self.connection.request(
+            action,
+            params=params)
+        return r.object
+
     def ex_get_node(self, id):
         """
         Fetch information about a node.
@@ -1628,10 +1844,11 @@ class AzureNodeDriver(NodeDriver):
         :rtype: :class:`.AzureIPAddress`
         """
 
-        if location is None and self.default_location:
-            location = self.default_location
-        else:
-            raise ValueError("location is required.")
+        if location is None:
+            if self.default_location:
+                location = self.default_location
+            else:
+                raise ValueError("location is required.")
 
         target = "/subscriptions/%s/resourceGroups/%s/" \
                  "providers/Microsoft.Network/publicIPAddresses/%s" \
@@ -1653,6 +1870,26 @@ class AzureNodeDriver(NodeDriver):
                                     method='PUT')
         return self._to_ip_address(r.object)
 
+    def ex_delete_public_ip(self, public_ip):
+        """
+        Delete a public ip address resource.
+
+        :param public_ip: Public ip address resource to delete
+        :type public_ip: `.AzureIPAddress`
+        """
+        # NOTE: This operation requires API version 2018-11-01 so
+        # "ex_delete_resource" won't for for deleting an IP address
+        resource = public_ip.id
+        r = self.connection.request(
+            resource,
+            method='DELETE',
+            params={
+                'api-version': '2019-06-01'
+            },
+        )
+
+        return r.status in [200, 202, 204]
+
     def ex_create_network_interface(self, name, subnet, resource_group,
                                     location=None, public_ip=None):
         """
@@ -1737,7 +1974,7 @@ class AzureNodeDriver(NodeDriver):
             resource = resource.id
         r = self.connection.request(
             resource,
-            params={"api-version": RESOURCE_API_VERSION})
+            params={"api-version": '2018-06-01'})
         if replace:
             r.object["tags"] = tags
         else:
@@ -1745,10 +1982,10 @@ class AzureNodeDriver(NodeDriver):
         self.connection.request(
             resource,
             data={"tags": r.object["tags"]},
-            params={"api-version": RESOURCE_API_VERSION},
+            params={"api-version": '2018-06-01'},
             method="PATCH")
 
-    def ex_start_node(self, node):
+    def start_node(self, node):
         """
         Start a stopped node.
 
@@ -1762,7 +1999,7 @@ class AzureNodeDriver(NodeDriver):
                                     method='POST')
         return r.object
 
-    def ex_stop_node(self, node, deallocate=True):
+    def stop_node(self, node, ex_deallocate=True):
         """
         Stop a running node.
 
@@ -1776,16 +2013,27 @@ class AzureNodeDriver(NodeDriver):
         were running.
         :type deallocate: ``bool``
         """
-
-        if deallocate:
+        if ex_deallocate:
             target = "%s/deallocate" % node.id
         else:
-            target = "%s/stop" % node.id
+            target = "%s/powerOff" % node.id
         r = self.connection.request(target,
                                     params={"api-version": "2015-06-15"},
                                     method='POST')
         return r.object
 
+    def ex_start_node(self, node):
+        # NOTE: This method is here for backward compatibility reasons after
+        # this method was promoted to be part of the standard compute API in
+        # Libcloud v2.7.0
+        return self.start_node(node=node)
+
+    def ex_stop_node(self, node, deallocate=True):
+        # NOTE: This method is here for backward compatibility reasons after
+        # this method was promoted to be part of the standard compute API in
+        # Libcloud v2.7.0
+        return self.stop_node(node=node, ex_deallocate=deallocate)
+
     def ex_get_storage_account_keys(self, resource_group, storage_account):
         """
         Get account keys required to access to a storage account
@@ -1903,9 +2151,8 @@ class AzureNodeDriver(NodeDriver):
                 keys["key1"],
                 host="%s.blob%s" % (storageAccount,
                                     self.connection.storage_suffix))
-            blobdriver.delete_object(blobdriver.get_object(blobContainer,
-                                                           blob))
-            return True
+            return blobdriver.delete_object(
+                blobdriver.get_object(blobContainer, blob))
         except ObjectDoesNotExistError:
             return True
 
@@ -1916,7 +2163,44 @@ class AzureNodeDriver(NodeDriver):
         kwargs["cloud_environment"] = self.cloud_environment
         return kwargs
 
-    def _to_node(self, data, fetch_nic=True):
+    def _fetch_power_state(self, data):
+        state = NodeState.UNKNOWN
+        try:
+            action = "%s/InstanceView" % (data["id"])
+            r = self.connection.request(action,
+                                        params={"api-version": "2015-06-15"})
+            for status in r.object["statuses"]:
+                if status["code"] in ["ProvisioningState/creating"]:
+                    state = NodeState.PENDING
+                    break
+                elif status["code"] == "ProvisioningState/deleting":
+                    state = NodeState.TERMINATED
+                    break
+                elif status["code"].startswith("ProvisioningState/failed"):
+                    state = NodeState.ERROR
+                    break
+                elif status["code"] == "ProvisioningState/updating":
+                    state = NodeState.UPDATING
+                    break
+                elif status["code"] == "ProvisioningState/succeeded":
+                    pass
+
+                if status["code"] == "PowerState/deallocated":
+                    state = NodeState.STOPPED
+                    break
+                elif status["code"] == "PowerState/stopped":
+                    state = NodeState.PAUSED
+                    break
+                elif status["code"] == "PowerState/deallocating":
+                    state = NodeState.PENDING
+                    break
+                elif status["code"] == "PowerState/running":
+                    state = NodeState.RUNNING
+        except BaseHTTPError:
+            pass
+        return state
+
+    def _to_node(self, data, fetch_nic=True, fetch_power_state=True):
         private_ips = []
         public_ips = []
         nics = data["properties"]["networkProfile"]["networkInterfaces"]
@@ -1939,36 +2223,20 @@ class AzureNodeDriver(NodeDriver):
                     pass
 
         state = NodeState.UNKNOWN
-        try:
-            action = "%s/InstanceView" % (data["id"])
-            r = self.connection.request(action,
-                                        params={"api-version": "2015-06-15"})
-            for status in r.object["statuses"]:
-                if status["code"] == "ProvisioningState/creating":
-                    state = NodeState.PENDING
-                    break
-                elif status["code"] == "ProvisioningState/deleting":
-                    state = NodeState.TERMINATED
-                    break
-                elif status["code"].startswith("ProvisioningState/failed"):
-                    state = NodeState.ERROR
-                    break
-                elif status["code"] == "ProvisioningState/updating":
-                    state = NodeState.UPDATING
-                    break
-                elif status["code"] == "ProvisioningState/succeeded":
-                    pass
-
-                if status["code"] == "PowerState/deallocated":
-                    state = NodeState.STOPPED
-                    break
-                elif status["code"] == "PowerState/deallocating":
-                    state = NodeState.PENDING
-                    break
-                elif status["code"] == "PowerState/running":
-                    state = NodeState.RUNNING
-        except BaseHTTPError:
-            pass
+        if fetch_power_state:
+            state = self._fetch_power_state(data)
+        else:
+            ps = data["properties"]["provisioningState"].lower()
+            if ps == "creating":
+                state = NodeState.PENDING
+            elif ps == "deleting":
+                state = NodeState.TERMINATED
+            elif ps == "failed":
+                state = NodeState.ERROR
+            elif ps == "updating":
+                state = NodeState.UPDATING
+            elif ps == "succeeded":
+                state = NodeState.RUNNING
 
         node = Node(data["id"],
                     data["name"],
@@ -1994,8 +2262,8 @@ class AzureNodeDriver(NodeDriver):
                                "maxDataDiskCount": data["maxDataDiskCount"]})
 
     def _to_nic(self, data):
-        return AzureNic(data["id"], data["name"], data["location"],
-                        data["properties"])
+        return AzureNic(data["id"], data.get("name"), data.get("location"),
+                        data.get("properties"))
 
     def _to_ip_address(self, data):
         return AzureIPAddress(data["id"], data["name"], data["properties"])
@@ -2011,7 +2279,8 @@ class AzureNodeDriver(NodeDriver):
     def _get_instance_vhd(self, name, ex_resource_group, ex_storage_account,
                           ex_blob_container="vhds"):
         n = 0
-        while True:
+        errors = []
+        while n < 10:
             try:
                 instance_vhd = "https://%s.blob%s" \
                                "/%s/%s-os_%i.vhd" \
@@ -2020,10 +2289,16 @@ class AzureNodeDriver(NodeDriver):
                                   ex_blob_container,
                                   name,
                                   n)
-                self._ex_delete_old_vhd(ex_resource_group, instance_vhd)
-                return instance_vhd
-            except LibcloudError:
-                n += 1
+                if self._ex_delete_old_vhd(ex_resource_group, instance_vhd):
+                    # We were able to remove it or it doesn't exist,
+                    # so we can use it.
+                    return instance_vhd
+            except LibcloudError as lce:
+                errors.append(str(lce))
+            n += 1
+        raise LibcloudError("Unable to find a name for a VHD to use for "
+                            "instance in 10 tries, errors were:\n  - %s" %
+                            ("\n  - ".join(errors)))
 
 
 def _split_blob_uri(uri):
