@@ -19,7 +19,6 @@ import base64
 import codecs
 import hmac
 import time
-import sys
 from hashlib import sha1
 
 from libcloud.utils.py3 import ET
@@ -76,7 +75,7 @@ class OSSResponse(XmlResponse):
 
     def success(self):
         i = int(self.status)
-        return i >= 200 and i <= 299 or i in self.valid_response_codes
+        return 200 <= i <= 299 or i in self.valid_response_codes
 
     def parse_body(self):
         """
@@ -91,7 +90,7 @@ class OSSResponse(XmlResponse):
                 body = ET.XML(self.body.encode('utf-8'), parser=parser)
             else:
                 body = ET.XML(self.body)
-        except:
+        except Exception:
             raise MalformedResponseError('Failed to parse XML',
                                          body=self.body,
                                          driver=self.connection.driver)
@@ -207,6 +206,8 @@ class OSSConnection(ConnectionUserAndKey):
             path = '/%s%s' % (self._container.name, self.action)
         else:
             path = self.action
+
+        # pylint: disable=no-member
         params['Signature'] = self._get_auth_signature(
             method=self.method, headers=headers, params=params,
             expires=params['Expires'], secret_key=self.key, path=path,
@@ -278,38 +279,30 @@ class OSSStorageDriver(StorageDriver):
         raise LibcloudError('Unexpected status code: %s' % (response.status),
                             driver=self)
 
-    def list_container_objects(self, container, ex_prefix=None):
-        """
-        Return a list of objects for the given container.
-
-        :param container: Container instance.
-        :type container: :class:`Container`
-
-        :keyword ex_prefix: Only return objects starting with ex_prefix
-        :type ex_prefix: ``str``
-
-        :return: A list of Object instances.
-        :rtype: ``list`` of :class:`Object`
-        """
-        return list(self.iterate_container_objects(container,
-                    ex_prefix=ex_prefix))
-
-    def iterate_container_objects(self, container, ex_prefix=None):
+    def iterate_container_objects(self, container, prefix=None,
+                                  ex_prefix=None):
         """
         Return a generator of objects for the given container.
 
         :param container: Container instance
         :type container: :class:`Container`
 
-        :keyword ex_prefix: Only return objects starting with ex_prefix
+        :keyword prefix: Only return objects starting with prefix
+        :type prefix: ``str``
+
+        :keyword ex_prefix: (Deprecated.) Only return objects starting with
+                            ex_prefix
         :type ex_prefix: ``str``
 
         :return: A generator of Object instances.
         :rtype: ``generator`` of :class:`Object`
         """
+        prefix = self._normalize_prefix_argument(prefix, ex_prefix)
+
         params = {}
-        if ex_prefix:
-            params['prefix'] = ex_prefix
+
+        if prefix:
+            params['prefix'] = prefix
 
         last_key = None
         exhausted = False
@@ -467,7 +460,8 @@ class OSSStorageDriver(StorageDriver):
             pass
         return self._put_object(container=container, object_name=object_name,
                                 extra=extra, method=method, query_args=params,
-                                stream=iterator, verify_hash=False)
+                                stream=iterator, verify_hash=False,
+                                headers=headers)
 
     def delete_object(self, obj):
         object_path = self._get_object_path(obj.container, obj.name)
@@ -584,11 +578,11 @@ class OSSStorageDriver(StorageDriver):
 
     def _put_object(self, container, object_name, method='PUT',
                     query_args=None, extra=None, file_path=None,
-                    stream=None, verify_hash=False):
+                    stream=None, verify_hash=False, headers=None):
         """
         Create an object and upload data using the given function.
         """
-        headers = {}
+        headers = headers or {}
         extra = extra or {}
 
         content_type = extra.get('content_type', None)
@@ -685,11 +679,10 @@ class OSSStorageDriver(StorageDriver):
             # Commit the chunk info and complete the upload
             etag = self._commit_multipart(object_path, upload_id, chunks,
                                           container=container)
-        except Exception:
-            exc = sys.exc_info()[1]
+        except Exception as e:
             # Amazon provides a mechanism for aborting an upload.
             self._abort_multipart(object_path, upload_id, container=container)
-            raise exc
+            raise e
 
         # Modify the response header of the first request. This is used
         # by other functions once the callback is done
