@@ -80,6 +80,21 @@ class DigitalOcean_v2_Tests(LibcloudTestCase):
         self.assertEqual(size.name, '1gb')
         self.assertEqual(size.ram, 1024)
 
+    def test_list_sizes_filter_by_location_success(self):
+        location = self.driver.list_locations()[1]
+        sizes = self.driver.list_sizes(location=location)
+        self.assertTrue(len(sizes) >= 1)
+
+        size = sizes[0]
+        self.assertTrue(size.id is not None)
+        self.assertEqual(size.name, '512mb')
+        self.assertTrue(location.id in size.extra['regions'])
+
+        location = self.driver.list_locations()[1]
+        location.id = 'doesntexist'
+        sizes = self.driver.list_sizes(location=location)
+        self.assertEqual(len(sizes), 0)
+
     def test_list_locations_success(self):
         locations = self.driver.list_locations()
         self.assertTrue(len(locations) == 2)
