@@ -43,7 +43,7 @@ from libcloud.storage.types import ContainerIsNotEmptyError
 from libcloud.storage.types import InvalidContainerNameError
 from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import ObjectHashMismatchError
-from libcloud.storage.drivers.s3 import BaseS3Connection
+from libcloud.storage.drivers.s3 import BaseS3Connection, S3SignatureV4Connection
 from libcloud.storage.drivers.s3 import S3StorageDriver, S3USWestStorageDriver
 from libcloud.storage.drivers.s3 import CHUNK_SIZE
 from libcloud.utils.py3 import b
@@ -52,9 +52,10 @@ from libcloud.test import MockHttp  # pylint: disable-msg=E0611
 from libcloud.test import unittest, make_response, generate_random_data
 from libcloud.test.file_fixtures import StorageFileFixtures  # pylint: disable-msg=E0611
 from libcloud.test.secrets import STORAGE_S3_PARAMS
+from libcloud.test.storage.base import BaseRangeDownloadMockHttp
 
 
-class S3MockHttp(MockHttp):
+class S3MockHttp(BaseRangeDownloadMockHttp, unittest.TestCase):
 
     fixtures = StorageFileFixtures('s3')
     base_headers = {}
@@ -327,6 +328,34 @@ class S3MockHttp(MockHttp):
                 headers,
                 httplib.responses[httplib.OK])
 
+    def _foo_bar_container_foo_bar_object_range(self, method, url, body, headers):
+        # test_download_object_range_success
+        body = '0123456789123456789'
+
+        self.assertTrue('Range' in headers)
+        self.assertEqual(headers['Range'], 'bytes=5-6')
+
+        start_bytes, end_bytes = self._get_start_and_end_bytes_from_range_str(headers['Range'], body)
+
+        return (httplib.PARTIAL_CONTENT,
+                body[start_bytes:end_bytes + 1],
+                headers,
+                httplib.responses[httplib.PARTIAL_CONTENT])
+
+    def _foo_bar_container_foo_bar_object_range_stream(self, method, url, body, headers):
+        # test_download_object_range_as_stream_success
+        body = '0123456789123456789'
+
+        self.assertTrue('Range' in headers)
+        self.assertEqual(headers['Range'], 'bytes=4-6')
+
+        start_bytes, end_bytes = self._get_start_and_end_bytes_from_range_str(headers['Range'], body)
+
+        return (httplib.PARTIAL_CONTENT,
+                body[start_bytes:end_bytes + 1],
+                headers,
+                httplib.responses[httplib.PARTIAL_CONTENT])
+
     def _foo_bar_container_foo_bar_object_NO_BUFFER(self, method, url, body, headers):
         # test_download_object_data_is_not_buffered_in_memory
         body = generate_random_data(1000)
@@ -335,27 +364,6 @@ class S3MockHttp(MockHttp):
                 headers,
                 httplib.responses[httplib.OK])
 
-    def _foo_bar_container_foo_test_upload_INVALID_HASH1(self, method, url,
-                                                         body, headers):
-        body = ''
-        headers = {}
-        headers['etag'] = '"foobar"'
-        # test_upload_object_invalid_hash1
-        return (httplib.OK,
-                body,
-                headers,
-                httplib.responses[httplib.OK])
-
-    def _foo_bar_container_foo_test_upload_INVALID_HASH2(self, method, url,
-                                                         body, headers):
-        # test_upload_object_invalid_hash2
-        body = ''
-        headers = {'etag': '"hash343hhash89h932439jsaa89"'}
-        return (httplib.OK,
-                body,
-                headers,
-                httplib.responses[httplib.OK])
-
     def _foo_bar_container_foo_test_upload(self, method, url, body, headers):
         # test_upload_object_success
         body = ''
@@ -401,6 +409,18 @@ class S3Tests(unittest.TestCase):
         except OSError:
             pass
 
+    def test_clean_object_name(self):
+        # Ensure ~ is not URL encoded
+        # See https://github.com/apache/libcloud/issues/1452 for details
+        cleaned = self.driver._clean_object_name(name='valid')
+        self.assertEqual(cleaned, 'valid')
+
+        cleaned = self.driver._clean_object_name(name='valid/~')
+        self.assertEqual(cleaned, 'valid/~')
+
+        cleaned = self.driver._clean_object_name(name='valid/~%foo ')
+        self.assertEqual(cleaned, 'valid/~%25foo%20')
+
     def test_invalid_credentials(self):
         self.mock_response_klass.type = 'UNAUTHORIZED'
         try:
@@ -497,7 +517,7 @@ class S3Tests(unittest.TestCase):
         container = Container(name='test_container', extra={},
                               driver=self.driver)
         objects = self.driver.list_container_objects(container=container,
-                                                     ex_prefix='test_prefix')
+                                                     prefix='test_prefix')
         self.assertEqual(len(objects), 1)
 
         obj = [o for o in objects if o.name == '1.zip'][0]
@@ -520,6 +540,24 @@ class S3Tests(unittest.TestCase):
         container = self.driver.get_container(container_name='test1')
         self.assertTrue(container.name, 'test1')
 
+    def test_get_object_cdn_url(self):
+        self.mock_response_klass.type = 'get_object'
+        obj = self.driver.get_object(container_name='test2',
+                                     object_name='test')
+
+        # cdn urls can only be generated using a V4 connection
+        if issubclass(self.driver.connectionCls, S3SignatureV4Connection):
+            cdn_url = self.driver.get_object_cdn_url(obj, ex_expiry=12)
+            url = urlparse.urlparse(cdn_url)
+            query = urlparse.parse_qs(url.query)
+
+            self.assertEqual(len(query['X-Amz-Signature']), 1)
+            self.assertGreater(len(query['X-Amz-Signature'][0]), 0)
+            self.assertEqual(query['X-Amz-Expires'], ['43200'])
+        else:
+            with self.assertRaises(NotImplementedError):
+                self.driver.get_object_cdn_url(obj)
+
     def test_get_object_container_doesnt_exist(self):
         # This method makes two requests which makes mocking the response a bit
         # trickier
@@ -632,6 +670,38 @@ class S3Tests(unittest.TestCase):
                                              delete_on_failure=True)
         self.assertTrue(result)
 
+    def test_download_object_range_success(self):
+        container = Container(name='foo_bar_container', extra={},
+                              driver=self.driver)
+        obj = Object(name='foo_bar_object_range', size=19, hash=None, extra={},
+                     container=container, meta_data=None,
+                     driver=self.driver_type)
+        destination_path = self._file_path
+        result = self.driver.download_object_range(obj=obj,
+                                             destination_path=destination_path,
+                                             start_bytes=5,
+                                             end_bytes=7,
+                                             overwrite_existing=True,
+                                             delete_on_failure=True)
+        self.assertTrue(result)
+
+        with open(self._file_path, 'r') as fp:
+            content = fp.read()
+
+        self.assertEqual(content, '56')
+
+    def test_download_object_range_as_stream_success(self):
+        container = Container(name='foo_bar_container', extra={},
+                              driver=self.driver)
+        obj = Object(name='foo_bar_object_range_stream', size=19, hash=None, extra={},
+                     container=container, meta_data=None,
+                     driver=self.driver_type)
+        iterator = self.driver.download_object_range_as_stream(obj=obj,
+                                                               start_bytes=4,
+                                                               end_bytes=7)
+        content = exhaust_iterator(iterator)
+        self.assertEqual(content, b'456')
+
     def test_download_object_data_is_not_buffered_in_memory(self):
         # Test case which verifies that response.body attribute is not accessed
         # and as such, whole body response is not buffered into RAM
@@ -772,12 +842,11 @@ class S3Tests(unittest.TestCase):
         def upload_file(self, object_name=None, content_type=None,
                         request_path=None, request_method=None,
                         headers=None, file_path=None, stream=None):
-            return {'response': make_response(200),
+            headers = {'etag': '"foobar"'}
+            return {'response': make_response(200, headers=headers),
                     'bytes_transferred': 1000,
                     'data_hash': 'hash343hhash89h932439jsaa89'}
 
-        self.mock_response_klass.type = 'INVALID_HASH1'
-
         old_func = self.driver_type._upload_object
         self.driver_type._upload_object = upload_file
         file_path = os.path.abspath(__file__)
@@ -802,12 +871,11 @@ class S3Tests(unittest.TestCase):
         def upload_file(self, object_name=None, content_type=None,
                         request_path=None, request_method=None,
                         headers=None, file_path=None, stream=None):
-            return {'response': make_response(200, headers={'etag': 'woopwoopwoop'}),
+            headers = {'etag': '"hash343hhash89h932439jsaa89"'}
+            return {'response': make_response(200, headers=headers),
                     'bytes_transferred': 1000,
                     'data_hash': '0cc175b9c0f1b6a831c399e269772661'}
 
-        self.mock_response_klass.type = 'INVALID_HASH2'
-
         old_func = self.driver_type._upload_object
         self.driver_type._upload_object = upload_file
 
@@ -827,6 +895,31 @@ class S3Tests(unittest.TestCase):
         finally:
             self.driver_type._upload_object = old_func
 
+    def test_upload_object_invalid_hash_kms_encryption(self):
+        # Hash check should be skipped when AWS KMS server side encryption is
+        # used
+        def upload_file(self, object_name=None, content_type=None,
+                        request_path=None, request_method=None,
+                        headers=None, file_path=None, stream=None):
+            headers = {'etag': 'blahblah', 'x-amz-server-side-encryption': 'aws:kms'}
+            return {'response': make_response(200, headers=headers),
+                    'bytes_transferred': 1000,
+                    'data_hash': 'hash343hhash89h932439jsaa81'}
+
+        old_func = self.driver_type._upload_object
+        self.driver_type._upload_object = upload_file
+        file_path = os.path.abspath(__file__)
+        container = Container(name='foo_bar_container', extra={},
+                              driver=self.driver)
+        object_name = 'foo_test_upload'
+        try:
+            self.driver.upload_object(file_path=file_path, container=container,
+                                      object_name=object_name,
+                                      verify_hash=True)
+        finally:
+            self.driver_type._upload_object = old_func
+
+
     def test_upload_object_success(self):
         def upload_file(self, object_name=None, content_type=None,
                         request_path=None, request_method=None,
@@ -1068,8 +1161,8 @@ class S3Tests(unittest.TestCase):
 
         # Invalid region
         expected_msg = 'Invalid or unsupported region: foo'
-        self.assertRaisesRegexp(ValueError, expected_msg, S3StorageDriver,
-                                *self.driver_args, region='foo')
+        self.assertRaisesRegex(ValueError, expected_msg, S3StorageDriver,
+                               *self.driver_args, region='foo')
 
         # host argument still has precedence over reguin
         driver3  = S3StorageDriver(*self.driver_args, region='ap-south-1', host='host1.bar.com')
